@@ -16,6 +16,7 @@ from pdpipe.shared import (
 import pdpipe.cond as cond
 from pdpipe.types import ColumnsParamType
 from pdpipe.exceptions import FailedConditionError
+from pdpipe.cq import ColumnQualifier
 
 
 class ColDrop(ColumnsBasedPipelineStage):
@@ -26,7 +27,7 @@ class ColDrop(ColumnsBasedPipelineStage):
     columns : single label, list-like or callable
         The label, or an iterable of labels, of columns to drop. Alternatively,
         this parameter can be assigned a callable returning an iterable of
-        labels from an input pandas.DataFrame. See pdpipe.cq.
+        labels from an input pandas.DataFrame (see `pdpipe.cq`).
     errors : {‘ignore’, ‘raise’}, default ‘raise’
         If ‘ignore’, suppress error and existing labels are dropped.
 
@@ -82,12 +83,12 @@ class ValDrop(ColumnsBasedPipelineStage):
         The label, or an iterable of labels, of columns to check for the given
         values. Alternatively, this parameter can be assigned a callable
         returning an iterable of labels from an input pandas.DataFrame. See
-        pdpipe.cq. If set to None, all columns are checked.
-    exclude_columns : object, iterable or callable, optional
+        `pdpipe.cq`. If set to None, all columns are checked.
+    exclude_columns : label, iterable or callable, optional
         The label, or an iterable of labels, of columns to exclude, given the
         `columns` parameter. Alternatively, this parameter can be assigned a
         callable returning a labels iterable from an input pandas.DataFrame.
-        See pdpipe.cq. Optional. By default no columns are excluded.
+        See `pdpipe.cq`. Optional. By default no columns are excluded.
 
     Example
     -------
@@ -142,12 +143,12 @@ class ValKeep(ColumnsBasedPipelineStage):
         The label, or an iterable of labels, of columns to check for the given
         values. Alternatively, this parameter can be assigned a callable
         returning an iterable of labels from an input pandas.DataFrame. See
-        pdpipe.cq. If set to None, all columns are checked.
-    exclude_columns : object, iterable or callable, optional
+        `pdpipe.cq`. If set to None, all columns are checked.
+    exclude_columns : single label, iterable or callable, optional
         The label, or an iterable of labels, of columns to exclude, given the
         `columns` parameter. Alternatively, this parameter can be assigned a
         callable returning a labels iterable from an input pandas.DataFrame.
-        See pdpipe.cq. Optional. By default no columns are excluded.
+        See `pdpipe.cq`. Optional. By default no columns are excluded.
 
     Example
     -------
@@ -189,7 +190,7 @@ class ColRename(PdPipelineStage):
 
     Parameters
     ----------
-    rename_mapper : dict-like or function
+    rename_mapper : dict-like or callable
         Maps old column names to new ones.
 
     Example
@@ -200,21 +201,29 @@ class ColRename(PdPipelineStage):
            len initial
         1    8       a
         2    5       b
+
+        >>> def renamer(lbl: str):
+        ...    if lbl.startswith('n'):
+        ...       return 'foo'
+        ...    return lbl
+        >>> pdp.ColRename(renamer).apply(df)
+           foo char
+        1    8    a
+        2    5    b
     """
 
     _DEF_COLDRENAME_EXC_MSG = ("ColRename stage failed because not all columns"
                                " {} were found in input dataframe.")
 
-    def __init__(self, rename_mapper, **kwargs):
+    def __init__(self, rename_mapper: Union[Dict, Callable], **kwargs):
         self._rename_mapper = rename_mapper
         try:
             columns_str = _list_str(list(rename_mapper.keys()))
             mapper_repr = str(rename_mapper)
             keys_set = set(self._rename_mapper.keys())
-
-            def _tprec(df):
-                return keys_set.issubset(df.columns)
-        except AttributeError:
+            required_labels = list(keys_set)
+            _tprec = cond.HasAllColumns(required_labels)
+        except AttributeError:  # rename mapper is a callable
             mapper_repr = rename_mapper.__name__
             doc = rename_mapper.__doc__
             if doc is None:
@@ -224,9 +233,7 @@ class ColRename(PdPipelineStage):
                     f"by func {rename_mapper.__name__} with "
                     f"doc: {rename_mapper.__doc__}"
                 )
-
-            def _tprec(df):
-                return True
+            _tprec = cond.AlwaysTrue()
         try:
             suffix = 's' if len(rename_mapper) > 1 else ''
         except TypeError:
@@ -359,7 +366,7 @@ class FreqDrop(PdPipelineStage):
                              " found in input dataframe.")
     _DEF_FREQDROP_DESC = "Drop values with frequency < {} in column {}."
 
-    def __init__(self, threshold, column, **kwargs):
+    def __init__(self, threshold: int, column: str, **kwargs):
         self._threshold = threshold
         self._column = column
         super_kwargs = {
@@ -436,7 +443,7 @@ class ColReorder(PdPipelineStage):
 
 
 class RowDrop(ColumnsBasedPipelineStage):
-    """A pipeline stage that drop rows by callable conditions.
+    """A pipeline stage that drops rows by callable conditions.
 
     Parameters
     ----------
@@ -453,20 +460,20 @@ class RowDrop(ColumnsBasedPipelineStage):
         satisfying at least one of the conditions are dropped. If set to 'xor',
         rows satisfying exactly one of the conditions will be dropped. Set to
         'any' by default.
-    columns : str or iterable, optional
+    columns : single label, iterable or callable, optional
         The label, or an iterable of labels, of columns. Alternatively,
         this parameter can be assigned a callable returning an iterable of
-        labels from an input pandas.DataFrame. See pdpipe.cq. If given,
+        labels from an input pandas.DataFrame. See `pdpipe.cq`. If given,
         input conditions will be applied to the sub-dataframe made up of
         these columns to determine which rows to drop. Ignored if `conditions`
         is provided with a dict object. If `conditions` is a list and this
         parameter is not provided, all columns are checked (unless
         `exclude_columns` is additionally provided)
-    exclude_columns : object, iterable or callable, optional
+    exclude_columns : single label, iterable or callable, optional
         The label, or an iterable of labels, of columns to exclude, given the
         `columns` parameter. Alternatively, this parameter can be assigned a
         callable returning a labels iterable from an input pandas.DataFrame.
-        See pdpipe.cq. Optional. By default no columns are excluded.
+        See `pdpipe.cq`. Optional. By default no columns are excluded.
 
     Example
     -------
@@ -567,7 +574,7 @@ class Schematize(PdPipelineStage):
     Parameters
     ----------
     columns: sequence of labels
-        The dataframe schema to enfore on input dataframes.
+        The dataframe schema to enforce on input dataframes.
 
     Example
     -------
@@ -620,7 +627,7 @@ class DropDuplicates(ColumnsBasedPipelineStage):
         The label, or an iterable of labels, of columns to exclude, given the
         `columns` parameter. Alternatively, this parameter can be assigned a
         callable returning a labels iterable from an input pandas.DataFrame.
-        See pdpipe.cq. Optional. By default no columns are excluded.
+        See `pdpipe.cq`. Optional. By default no columns are excluded.
 
     Examples
     --------
@@ -654,10 +661,16 @@ class ColumnDtypeEnforcer(PdPipelineStage):
 
     Parameters
     ----------
-    column_to_dtype: dict of column name -> data type
+    column_to_dtype: dict of labels / ColumnQualifiers to dtypes
         Use {col: dtype, …}, where col is a column label and dtype is a
         numpy.dtype or Python type to cast one or more of the DataFrame’s
-        columns to column-specific types.
+        columns to column-specific types. Alternatively, you can provide
+        `ColumnQualifier` objects as keys. If at least one such key is present,
+        the lbl-to-dtype dict is dynamically inferred each time the pipeline
+        stage is applied (note that `ColumnQualifier` objects are fittable by
+        default, so to have column labels re-inferred after the first stage
+        application you'll have to set `fittable=False` for the
+        `ColumnQualifier` you use, see `pdpipe.cq`).
     errors: {‘raise’, ‘ignore’}, default ‘raise’
         Control raising of exceptions on invalid data for provided dtype.
         - raise : allow exceptions to be raised
@@ -671,6 +684,11 @@ class ColumnDtypeEnforcer(PdPipelineStage):
            num initial
         1  8.0       a
         2  5.0       b
+
+        >>> pdp.ColumnDtypeEnforcer({pdp.cq.StartWith('n'): float}).apply(df)
+           num initial
+        1  8.0       a
+        2  5.0       b
     """
 
     _DEF_COL_DTYPE_ENF_EXC_MSG = (
@@ -679,27 +697,53 @@ class ColumnDtypeEnforcer(PdPipelineStage):
 
     def __init__(
         self,
-        column_to_dtype: Dict[str, object],
+        column_to_dtype: Dict,
         errors: Optional[str] = 'raise',
         **kwargs: object,
     ) -> None:
-        self._column_to_dtype = column_to_dtype
+        # if none of the keys in column_to_dtype is a ColumnQualifier
+        if not any(isinstance(
+                x, ColumnQualifier) for x in column_to_dtype.keys()):
+            # its a static map; use it as is
+            self._column_to_dtype = column_to_dtype
+            keys_set = set(column_to_dtype.keys())
+
+            def _tprec(df: pandas.DataFrame) -> bool:
+                return keys_set.issubset(df.columns)
+        else:
+            # else, it's at least partly dynamic, and will have to infer it
+            # on run time
+            self._dynamic_column_to_dtype = column_to_dtype
+
+            def _tprec(df: pandas.DataFrame) -> bool:
+                return True
+        self._tprec = _tprec
         self._errors = errors
         columns_str = _list_str(list(column_to_dtype.keys()))
         suffix = 's' if len(column_to_dtype) > 1 else ''
-        keys_set = set(column_to_dtype.keys())
-
-        def _tprec(df: pandas.DataFrame) -> bool:
-            return keys_set.issubset(df.columns)
-        self._tprec = _tprec
         super_kwargs = {
             'exmsg': ColumnDtypeEnforcer._DEF_COL_DTYPE_ENF_EXC_MSG.format(
                 columns_str),
-            'desc': f"Renam column{suffix} with {column_to_dtype}",
+            'desc': f"Enforce column{suffix} dtype with {column_to_dtype}",
         }
         super_kwargs.update(**kwargs)
         super().__init__(**super_kwargs)
 
+    def _col_to_dtype_from_df(self, df: pandas.DataFrame) -> Dict:
+        try:
+            return self._column_to_dtype
+        except AttributeError:
+            column_to_dtype = {}
+            for k, dtype in self._dynamic_column_to_dtype.items():
+                try:
+                    column_to_dtype.update({
+                        lbl: dtype
+                        for lbl in k(df)
+                    })
+                except TypeError:  # k is not a callable
+                    column_to_dtype[k] = dtype
+            return column_to_dtype
+
     def _prec(self, df: pandas.DataFrame) -> bool:
         return self._tprec(df)
 
@@ -708,8 +752,9 @@ class ColumnDtypeEnforcer(PdPipelineStage):
         df: pandas.DataFrame,
         verbose: bool,
     ) -> pandas.DataFrame:
+        lbl_to_dtype = self._col_to_dtype_from_df(df)
         return df.astype(
-            dtype=self._column_to_dtype,
+            dtype=lbl_to_dtype,
             copy=True,
             errors=self._errors,
         )
@@ -724,14 +769,13 @@ class ConditionValidator(PdPipelineStage):
     objects, and checks that all these callable return True - meaning all
     defined conditions hold - for input dataframes.
 
-    Naturally, pdpipe Condition objects from the pdpipe.cond module can be
-    used.
+    Naturally, pdpipe `Condition` objects from the `pdpipe.cond` module can be used.
 
     Parameters
     ----------
     conditions : callable or list-like of callable
         The conditions to check for input dataframes. Naturally, pdpipe
-        Condition objects from the pdpipe.cond module can be used.
+        `Condition` objects from the `pdpipe.cond` module can be used.
     reducer : callable, optional
         The callable that reduces the list of boolean result to a single
         result. By default the built-in `all` function is used, so all
@@ -750,7 +794,12 @@ class ConditionValidator(PdPipelineStage):
     -------
         >>> import pandas as pd; import pdpipe as pdp;
         >>> df = pd.DataFrame([[1,4],[4,None],[1,11]], [1,2,3], ['a','b'])
-        >>> pdp.ConditionValidator(pdp.cond.HasNoMissingValues())(df)
+        >>> pdp.ConditionValidator(lambda df: len(df.columns) == 5).apply(df)
+        Traceback (most recent call last):
+           ...
+        pdpipe.exceptions.FailedConditionError: ConditionValidator stage failed; some conditions did not hold for the input dataframe!
+
+        >>> pdp.ConditionValidator(pdp.cond.HasNoMissingValues()).apply(df)
         Traceback (most recent call last):
            ...
         pdpipe.exceptions.FailedConditionError: ConditionValidator stage failed; some conditions did not hold for the input dataframe!
